<?php

/**
 *
 *    Copyright (C) 2017 onOffice GmbH
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Affero General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Affero General Public License for more details.
 *
 *    You should have received a copy of the GNU Affero General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

declare(strict_types=1);

namespace onOffice\WPlugin\DataView;

use onOffice\WPlugin\Types\LinksTypes;
use onOffice\WPlugin\Types\MovieLinkTypes;

/**
 *
 * @url http://www.onoffice.de
 * @copyright 2003-2017, onOffice(R) GmbH
 *
 * DO NOT MOVE OR RENAME - NAME AND/OR NAMESPACE MAY BE USED IN SERIALIZED DATA
 *
 */

class DataDetailView
	implements DataView
{
	/** */
	const ENABLE_SIMILAR_ESTATES = 'enablesimilarestates';

	/** */
	const PICTURES = 'pictures';

	/** */
	const FIELDS = 'fields';

	/** */
	const ADDRESSFIELDS = 'addressfields';

	/** @var string[] */
	private $_fields = [
		'objekttitel',
		'objektart',
		'objekttyp',
		'vermarktungsart',
		'plz',
		'ort',
		'bundesland',
		'objektnr_extern',
		'wohnflaeche',
		'grundstuecksflaeche',
		'nutzflaeche',
		'anzahl_zimmer',
		'anzahl_badezimmer',
		'kaufpreis',
		'kaltmiete',
		'objektbeschreibung',
		'lage',
		'ausstatt_beschr',
		'sonstige_angaben'
	];

	/** @var string[] */
	private $_addressFields = [
		'imageUrl',
		'Anrede',
		'Vorname',
		'Name',
		'Zusatz1',
		'Strasse',
		'Plz',
		'Ort',
		'Telefon1',
		'mobile',
		'defaultemail',
	];

	/** @var string[] */
	private $_defaultAddressFields = [
		'Anrede',
		'Vorname',
		'Name',
		'Zusatz1',
		'Strasse',
		'Plz',
		'Ort',
		'Telefon1',
		'mobile',
		'defaultemail',
		'imageUrl',
	];

	/** @var string[] */
	private $_pictureTypes = [];

	/** @var string */
	private $_template = '';

	/** @var bool */
	private $_accessControls = true;

	/** @var bool */
	private $_restrictAccess = true;

	/** @var string */
	private $_shortCodeForm = '';

	/** @var string */
	private $_expose = '';

	/** @var int */
	private $_pageId = 0;

	/** @var array */
	private $_pageIdsHaveDetailShortCode = [];

	/** @var bool */
	private $_showStatus = 0;

	/** @var int */
	private $_movieLinks = MovieLinkTypes::MOVIE_LINKS_NONE;

	/** @var string */
	private $_oguloLinks = LinksTypes::LINKS_DEACTIVATED;

	/** @var string */
	private $_objectLinks = LinksTypes::LINKS_DEACTIVATED;

	/** @var string */
	private $_links = LinksTypes::LINKS_DEACTIVATED;

	/** @var bool */
	private $_dataDetailViewActive = false;

	/** @var DataViewSimilarEstates */
	private $_pDataViewSimilarEstates = null;


	/**
	 *
	 */

	public function __construct()
	{
		$this->_pDataViewSimilarEstates = new DataViewSimilarEstates();
	}

	/**
	 *
	 */

	public function __wakeup()
	{
		if ($this->_pDataViewSimilarEstates === null) {
			$this->_pDataViewSimilarEstates = new DataViewSimilarEstates();
		}
	}

	/** @return array */
	public function getFields(): array
		{ return $this->_fields; }

	/** @return array */
	public function getPictureTypes(): array
		{ return $this->_pictureTypes; }

	/** @return bool */
	public function hasDetailView(): bool
		{ return $this->_accessControls; }

	/** @return bool */
	public function getViewRestrict(): bool
	{ return $this->_restrictAccess; }

	/** @return string */
	public function getTemplate(): string
		{ return $this->_template; }

	/** @return string */
	public function getShortCodeForm(): string
		{ return $this->_shortCodeForm;}

	/** @return string */
	public function getExpose(): string
		{ return $this->_expose; }

	/** @return string */
	public function getName(): string
		{ return 'detail'; }

	/** @return string[] */
	public function getAddressFields(): array
		{ return $this->_addressFields; }

	/** @return string[] */
	public function getDefaultAddressFields(): array
		{ return $this->_defaultAddressFields; }

	/** @return int */
	public function getPageId(): int
		{ return $this->_pageId; }

	/** @param array $fields */
	public function setFields(array $fields)
		{ $this->_fields = $fields; }

	/** @param array $pictureTypes */
	public function setPictureTypes(array $pictureTypes)
		{ $this->_pictureTypes = $pictureTypes; }

	/** @param bool $accessControl */
	public function setHasDetailView(bool $accessControl)
		{ $this->_accessControls = $accessControl; }
    
	/** @param bool $accessControl */
	public function setHasDetailViewRestrict(bool $restrictAccess)
	{ $this->_restrictAccess = $restrictAccess; }

	/** @param string $template */
	public function setTemplate(string $template)
		{ $this->_template = $template; }

	/** @param string $shortCodeForm */
	public function setShortCodeForm(string $shortCodeForm)
	{$this->_shortCodeForm = $shortCodeForm;}

	/** @param string $expose */
	public function setExpose(string $expose)
		{ $this->_expose = $expose; }

	/** @param int $pageId */
	public function setPageId(int $pageId)
		{ $this->_pageId = $pageId; }

	/** @var string[] $addressFields */
	public function setAddressFields(array $addressFields)
		{ $this->_addressFields = $addressFields; }
	/** @return int */
	public function getMovieLinks(): int
		{ return $this->_movieLinks; }

	/** @return string */
	public function getOguloLinks(): string
		{ return $this->_oguloLinks; }

	/** @return string */
	public function getObjectLinks(): string
	{ return $this->_objectLinks; }

	/** @return string */
	public function getLinks(): string
	{ return $this->_links; }

	/** @return DataViewSimilarEstates */
	public function getDataViewSimilarEstates(): DataViewSimilarEstates
	{ return $this->_pDataViewSimilarEstates; }

	/** @param DataViewSimilarEstates $pDataViewSimilarEstates */
	public function setDataViewSimilarEstates(DataViewSimilarEstates $pDataViewSimilarEstates)
	{ $this->_pDataViewSimilarEstates = $pDataViewSimilarEstates; }

	/** @return bool */
	public function getDataDetailViewActive(): bool
	{ return $this->_dataDetailViewActive; }

	/** @param bool $dataDetailViewActive */
	public function setDataDetailViewActive(bool $dataDetailViewActive)
	{ $this->_dataDetailViewActive = $dataDetailViewActive; }

	/** @param int $movieLinks */
	public function setMovieLinks(int $movieLinks)
		{ $this->_movieLinks = $movieLinks; }

	/** @param string $oguloLinks */
	public function setOguloLinks(string $oguloLinks)
	{ $this->_oguloLinks = $oguloLinks; }

	/** @param string $objectLink */
	public function setObjectLinks(string $objectLink)
	{ $this->_objectLinks = $objectLink; }

	/** @param string $links */
	public function setLinks(string $links)
	{ $this->_links = $links; }

	/** @return bool */
	public function getRandom(): bool
		{  return false; }

	/** @return bool */
	public function getShowStatus(): bool
	{ return (bool) $this->_showStatus; }

	/** @param bool $status */
	public function setShowStatus(bool $status)
	{ $this->_showStatus = $status; }

	/** @return array */
	public function getPageIdsHaveDetailShortCode(): array
		{ return $this->_pageIdsHaveDetailShortCode; }

	/** @param int $pageId */
	public function addToPageIdsHaveDetailShortCode(int $pageId)
		{ $this->_pageIdsHaveDetailShortCode[$pageId] = $pageId; }

	/** @param int $pageId */
	public function removeFromPageIdsHaveDetailShortCode(int $pageId)
		{ unset($this->_pageIdsHaveDetailShortCode[$pageId]); }
}
